import os as __os
import signal as __sig
import functools as __fn
import fileinput as __file
from detacher import detach as __detach

__all__ = {"daemon", "set_logfile", "remove_logfile", "end_daemon"}

__JETSAM_LOG = "/tmp/jetsam.log"


def daemon(func) -> None:
    """
    Decorator to detach function from Python interpreter
    Spawns a new python interpreter to run function within
    """

    @__fn.wraps(func)
    def wrap():
        __detach(func, func.__name__, __JETSAM_LOG)

    return wrap


def set_logfile(logfile: str) -> None:
    globals()["__JETSAM_LOG"] = logfile


def remove_logfile() -> None:
    __os.remove(globals()["__JETSAM_LOG"])


def end_daemon(func) -> None:
    global __JETSAM_LOG
    with __file.input(__JETSAM_LOG, inplace=True) as f:
        for line in f:
            (func_name, pid, state) = line.strip("\n").split(":")
            if func_name == func.__name__ and "running" == state:
                __os.kill(int(pid), __sig.SIGKILL)
                print(f"{func_name}:{pid}:dead")
            else:
                print(line, end="")



#  ▐▄▄▄▄▄▄ .▄▄▄▄▄.▄▄ ·  ▄▄▄· • ▌ ▄ ·.
#   ·██▀▄.▀·•██  ▐█ ▀. ▐█ ▀█ ·██ ▐███▪
# ▪▄ ██▐▀▀▪▄ ▐█.▪▄▀▀▀█▄▄█▀▀█ ▐█ ▌▐▌▐█·
# ▐▌▐█▌▐█▄▄▌ ▐█▌·▐█▄▪▐█▐█ ▪▐▌██ ██▌▐█▌
#  ▀▀▀• ▀▀▀  ▀▀▀  ▀▀▀▀  ▀  ▀ ▀▀  █▪▀▀▀
from setuptools import setup, Extension
import platform

with open("README.md", "r") as f:
    long_desc = f.read()

define = "__BOGUS__"
system = platform.system()

if system == "Darwin":
    define = "_DARWIN_C_SOURCE"
elif system == "Linux":
    define = "__USE_MISC"


setup(
    name="jetsam",
    version="0.2.1",
    description="Jettison functions with jetsam!",
    long_description=long_desc,
    long_description_content_type="text/markdown",
    author="Tony B",
    author_email="tony@ballast.dev",
    license="MIT",
    license_files=["LICENSE"],
    ext_modules=[
        Extension(
            name="detacher",
            sources=["src/detach.c", "src/log.c"],
            include_dirs=["src/"],  # this is broken, need MANIFEST.in
            define_macros=[(define, None)],
        )
    ],
    package_dir={"": "src/"},
    # since package_dir is set, no automatic discovery
    py_modules=["jetsam"],
    project_urls={"Source Code": "https://gitlab.com/ballast-dev/jetsam"},
    python_requires=">=3.8",
)
